import pickle
import motmetrics as mm



class MOTEvaluator:
    """Wrapper for the `motmetrics` package for easy evaluation of a tracker.

    Typical usage would be to call `update` every frame with the ground truth and tracker results
    and then finally call `evaluate` in order to visualise the results.
    """

    def __init__(self, iou_threshold):
        """
        :param iou_threshold: If the intersection over union between object and ground truth is
        smaller than this value, then the object is treated as not matching the ground truth.
        """
        self.acc = mm.MOTAccumulator(auto_id=True)
        self.iou_threshold = iou_threshold

    def update(self, tracker_data, gt_data):
        """For the current frame, provide an list of IDs and bounding boxes from both the ground
        truth and tracker, and update the evaluation object with this data.  Update this with
        every frame's information.

        :param tracker_data: Tuple of two lists (tracker_ids, tracker_bboxes) containing the IDs of
        each tracker object in the first list. The second list is the corresponding bounding boxes,
        as a list of [x, y, w, h] lists.
        :param gt_data: Tuple of two lists (gt_ids, gt_bboxes) containing the IDs of each ground
        truth object in the first list. The second list is the corresponding bounding boxes, as a
        list of [x, y, w, h] lists.
        :raises AssertionError: If the supplied input arguments are not correct length (2).
        """
        assert len(tracker_data) == 2, "Incorrect length of tracker_data. It should be a list of\
                                        two lists tracker_ids, tracker_centroids"
        assert len(gt_data) == 2, "Incorrect length of gt_data. It should be a list\
                                   of two lists gt_ids, gt_centroids "
        tracker_ids, tracker_bboxes = tracker_data
        gt_ids, gt_bboxes = gt_data

        distance_matrix = mm.distances.iou_matrix(gt_bboxes, tracker_bboxes,
                                                  max_iou=self.iou_threshold)

        self.acc.update(gt_ids,  # Ground truth objects in this frame
                        tracker_ids,  # Detector hypotheses in this frame
                        distance_matrix)

    def evaluate(self, print_output=True, from_frame=0):
        """Calculates MOT challenge metrics for the evaluator across all frames it was updated
        with. "Full" shows across all frames and, if `from_frame` is supplied then the partial
        result can be shown only evaluating from this frame to the last frame.

        :param print_output: Whether to print formatted results to standard output.
        :param from_frame: An integer frame number. Used to also show result over a subset of
        frames from this frame number to the last frame number. The frame number is indexed by
        the order the evaluator instance was updated.
        :returns: A pandas dataframe object showing the evaluation results.
        """
        metrics_host = mm.metrics.create()
        summary = metrics_host.compute_many(
            [self.acc, self.acc.events.loc[from_frame:]],
            metrics=mm.metrics.motchallenge_metrics,
            names=["full", "part"])
        strsummary = mm.io.render_summary(
            summary,
            formatters=metrics_host.formatters,
            namemap=mm.io.motchallenge_metric_names
        )
        if print_output:
            print(strsummary)

        return summary

    def get_data(self):
        """Simply returns a pandas dataframe containing all the data stored in the evaluator so far.
        :returns: A pandas dataframe object showing all the data stored in the
        """
        dataframe = self.acc.events
        # print(df[df.Type != 'RAW']) # Useful for filtering to matches.
        return dataframe


def example():
    # Load data (deserialize)
    with open('tracker_pathological_example.pickle', 'rb') as handle:
        tracker_data = pickle.load(handle)
    evaluator = MOTEvaluator(iou_threshold=0.5)
    for data in tracker_data:
        (tracker_data, gt_data) = (data["tracker_data"], data["gt_data"])
        print(tracker_data, gt_data)
        evaluator.update(tracker_data, gt_data)

    evaluation = evaluator.evaluate(from_frame=0)
    mota = evaluation.mota.values[1]  # get the partial result
    print("Mota using equation is:")
    print(1 - (
            evaluation.num_misses + evaluation.num_false_positives + evaluation.num_switches)
          / evaluation.num_unique_objects)
    print("but MOTA reported is ", mota)

if __name__ == "__main__":
    example()
